@@ -13,12 +13,14 @@ use super::date::DateType;
 use super::datetime::DateTimeType;
 use super::float::FloatType;
 use super::integer::IntType;
+use super::interval::IntervalType;
 use super::null::NullType;
 use super::optional::OptionType;
 use super::range::RangeType;
 use super::text::TextType;
 use super::time::TimeType;
 use super::undefined::UndefType;
+use super::varargs::VarargsType;
 use super::variant::VariantType;
 
 dyn_clone::clone_trait_object!(DataType);
@@ -286,6 +288,15 @@ pub trait DataType: DynClone {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `= [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform `!=' or `<>` operator with
     /// between current DataType and any one of them
     ///
@@ -294,14 +305,32 @@ pub trait DataType: DynClone {
         vec![]
     }
 
-    /// Return a list of types that it's possible to perform `<=>' operator with
+    /// Return a list of types that it's possible to perform `!= [ALL|ANY|SOME]' operator with
     /// between current DataType and any one of them
     ///
     /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_bang_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
+    /// Return a list of types that it's possible to perform `<=>' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always IntType
     fn can_perform_null_safe_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `<=> [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always IntType
+    #[allow(unused_variables)]
+    fn can_perform_group_null_safe_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform `>' operator with
     /// between current DataType and any one of them
     ///
@@ -310,6 +339,15 @@ pub trait DataType: DynClone {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `> [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_gt_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform `>=' operator with
     /// between current DataType and any one of them
     ///
@@ -318,6 +356,15 @@ pub trait DataType: DynClone {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `>= [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_gte_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform `<' operator with
     /// between current DataType and any one of them
     ///
@@ -326,6 +373,15 @@ pub trait DataType: DynClone {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `< [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_lt_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform `=<' operator with
     /// between current DataType and any one of them
     ///
@@ -334,6 +390,15 @@ pub trait DataType: DynClone {
         vec![]
     }
 
+    /// Return a list of types that it's possible to perform `<= [ALL|ANY|SOME]' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    #[allow(unused_variables)]
+    fn can_perform_group_lte_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
     /// Return a list of types that it's possible to perform unary `NOT' operator with
     /// between current DataType and any one of them
     fn can_perform_not_op(&self) -> bool {
@@ -400,6 +465,30 @@ pub trait DataType: DynClone {
     fn can_perform_explicit_cast_op_to(&self) -> Vec<Box<dyn DataType>> {
         vec![]
     }
+
+    /// Return a list of types that it's possible to perform unary `LIKE' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    fn can_perform_like_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
+    /// Return a list of types that it's possible to perform unary `GLOB' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    fn can_perform_glob_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
+
+    /// Return a list of types that it's possible to perform unary `REGEXP' operator with
+    /// between current DataType and any one of them
+    ///
+    /// No need to define the result type, it always BoolType
+    fn can_perform_regexp_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![]
+    }
 }
 
 impl dyn DataType {
@@ -448,6 +537,11 @@ impl dyn DataType {
         self.as_any().downcast_ref::<DateTimeType>().is_some()
     }
 
+    /// Return true if this type is [`IntervalType`]
+    pub fn is_interval(&self) -> bool {
+        self.as_any().downcast_ref::<IntervalType>().is_some()
+    }
+
     /// Return true if this type is [`ArrayType`]
     pub fn is_array(&self) -> bool {
         self.as_any().downcast_ref::<ArrayType>().is_some()
@@ -490,9 +584,9 @@ impl dyn DataType {
         self.as_any().downcast_ref::<OptionType>().is_some()
     }
 
-    /// Return true if this type is [`VariantType`]
+    /// Return true if this type is [`VarargsType`]
     pub fn is_varargs(&self) -> bool {
-        self.as_any().downcast_ref::<VariantType>().is_some()
+        self.as_any().downcast_ref::<VarargsType>().is_some()
     }
 
     /// Return true if this type is [`CompositeType`]
