@@ -3,6 +3,7 @@ use std::any::Any;
 use crate::expression::Expr;
 use crate::expression::StringExpr;
 use crate::format_checker::is_valid_time_format;
+use crate::types::array::ArrayType;
 
 use super::base::DataType;
 
@@ -26,26 +27,50 @@ impl DataType for TimeType {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn can_perform_bang_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_bang_eq_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn can_perform_gt_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_gt_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn can_perform_gte_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_gte_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn can_perform_lt_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_lt_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn can_perform_lte_op_with(&self) -> Vec<Box<dyn DataType>> {
         vec![Box::new(TimeType)]
     }
 
+    fn can_perform_group_lte_op_with(&self) -> Vec<Box<dyn DataType>> {
+        vec![Box::new(ArrayType::new(Box::new(TimeType)))]
+    }
+
     fn has_implicit_cast_from(&self, expr: &Box<dyn Expr>) -> bool {
         if let Some(string_expr) = expr.as_any().downcast_ref::<StringExpr>() {
             return is_valid_time_format(&string_expr.value);
