@@ -3,7 +3,9 @@ use std::cmp::Ordering;
 use std::fmt;
 
 use dyn_clone::DynClone;
-use gitql_ast::types::base::DataType;
+use gitql_ast::operator::GroupComparisonOperator;
+use gitql_ast::types::DataType;
+use gitql_ast::Interval;
 
 use super::array::ArrayValue;
 use super::boolean::BoolValue;
@@ -12,6 +14,7 @@ use super::date::DateValue;
 use super::datetime::DateTimeValue;
 use super::float::FloatValue;
 use super::integer::IntValue;
+use super::interval::IntervalValue;
 use super::null::NullValue;
 use super::range::RangeValue;
 use super::text::TextValue;
@@ -39,128 +42,112 @@ pub trait Value: DynClone {
     /// Return the current value as dynamic [`Any`]
     fn as_any(&self) -> &dyn Any;
 
-    /// Perform unary `=` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `=` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn add_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `-` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `-` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn sub_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `*` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `*` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn mul_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `/` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `/` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn div_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `%` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `%` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn rem_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `^` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `^` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn caret_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `|` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `|` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn or_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `&` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `&` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn and_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `#` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `#` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn xor_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `||` or `OR` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `||` or `OR` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn logical_or_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `&&` or `AND` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `&&` or `AND` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn logical_and_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `XOR` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `XOR` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn logical_xor_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `<<` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `<<` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn shl_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `>>` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `>>` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn shr_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `[I]` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `[I]` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn index_op(&self, index: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `[S:E]` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `[S:E]` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn slice_op(
@@ -171,90 +158,176 @@ pub trait Value: DynClone {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `=` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `=` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn eq_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `!=` or `<>` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `= [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_eq_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `!=` or `<>` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn bang_eq_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `<=>` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `!= or <> [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_bang_eq_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `<=>` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn null_safe_eq_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `>` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `<=> [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_null_safe_eq_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `>` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn gt_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `>=` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `> [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_gt_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `>=` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn gte_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `<` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `>= [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_gte_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `<` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn lt_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `<=` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `< [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_lt_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `<=` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn lte_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `NOT` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `<= [ALL|ANY|SOME]` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn group_lte_op(
+        &self,
+        other: &Box<dyn Value>,
+        group_op: &GroupComparisonOperator,
+    ) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform unary `NOT` operator and return new [`Value`] represent the result or Exception message as [`String`]
     fn not_op(&self) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `-` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `-` operator and return new [`Value`] represent the result or Exception message as [`String`]
     fn neg_op(&self) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform unary `!` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform unary `!` operator and return new [`Value`] represent the result or Exception message as [`String`]
     fn bang_op(&self) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform `@>` operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform `@>` operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn contains_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
         Err("Unsupported operator for this type".to_string())
     }
 
-    /// Perform Cast operator and return new [`Value`] represent the old value after casted
-    /// or Exception message as [`String`]
+    /// Perform `LIKE` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn like_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform `GLOB` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn glob_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform `REGEXP` operator and return new [`Value`] represent the result or Exception message as [`String`]
+    #[allow(unused_variables)]
+    #[allow(clippy::borrowed_box)]
+    fn regexp_op(&self, other: &Box<dyn Value>) -> Result<Box<dyn Value>, String> {
+        Err("Unsupported operator for this type".to_string())
+    }
+
+    /// Perform Cast operator and return new [`Value`] represent the result or Exception message as [`String`]
     #[allow(unused_variables)]
     #[allow(clippy::borrowed_box)]
     fn cast_op(&self, target_type: &Box<dyn DataType>) -> Result<Box<dyn Value>, String> {
@@ -333,7 +406,7 @@ impl dyn Value {
     /// or None if this type it's called from wrong [`Value`]
     pub fn as_date(&self) -> Option<i64> {
         if let Some(date_value) = self.as_any().downcast_ref::<DateValue>() {
-            return Some(date_value.value);
+            return Some(date_value.timestamp);
         }
         None
     }
@@ -347,7 +420,7 @@ impl dyn Value {
     /// or None if this type it's called from wrong [`Value`]
     pub fn as_time(&self) -> Option<String> {
         if let Some(time_value) = self.as_any().downcast_ref::<DateValue>() {
-            return Some(time_value.value.to_string());
+            return Some(time_value.timestamp.to_string());
         }
         None
     }
@@ -366,11 +439,33 @@ impl dyn Value {
         None
     }
 
+    /// Return true if this value is [`IntervalValue`]
+    pub fn is_interval(&self) -> bool {
+        self.as_any().downcast_ref::<IntervalValue>().is_some()
+    }
+
+    /// Return List of [`Interval`] represent the value of [`IntervalValue`]
+    /// or None if this type it's called from wrong [`Value`]
+    pub fn as_interval(&self) -> Option<Interval> {
+        if let Some(interval_value) = self.as_any().downcast_ref::<IntervalValue>() {
+            return Some(interval_value.interval.clone());
+        }
+        None
+    }
+
     /// Return true if this value is [`ArrayValue`]
     pub fn is_array(&self) -> bool {
         self.as_any().downcast_ref::<ArrayValue>().is_some()
     }
 
+    /// Return true if this value is [`ArrayValue`] and element type match condition
+    pub fn is_array_of(&self, condition: fn(&Box<dyn DataType>) -> bool) -> bool {
+        if let Some(array) = self.as_any().downcast_ref::<ArrayValue>() {
+            return condition(&array.base_type);
+        }
+        false
+    }
+
     /// Return List of [`Value`] represent the inner values of [`ArrayValue`]
     /// or None if this type it's called from wrong [`Value`]
     pub fn as_array(&self) -> Option<Vec<Box<dyn Value>>> {
