@@ -7,6 +7,8 @@ pub mod token;
 pub mod tokenizer;
 
 pub(crate) mod parse_cast;
+pub(crate) mod parse_comparisons;
 pub(crate) mod parse_function_call;
+pub(crate) mod parse_interval;
 pub(crate) mod parse_type;
 pub mod parser;
