@@ -63,6 +63,7 @@ pub enum TokenKind {
     Partition,
     First,
     Last,
+    Interval,
 
     // Values
     Symbol(String),
@@ -77,6 +78,10 @@ pub enum TokenKind {
     Infinity,
     NaN,
 
+    All,
+    Some,
+    Any,
+
     Greater,
     GreaterEqual,
     Less,
@@ -177,19 +182,25 @@ impl Display for TokenKind {
             TokenKind::Nulls => "NULLS",
             TokenKind::First => "FIRST",
             TokenKind::Last => "LAST",
+            TokenKind::Interval => "INTERVAL",
 
             // Values
             TokenKind::Symbol(literal) => literal,
             TokenKind::GlobalVariable(literal) => literal,
             TokenKind::String(string) => string,
-            TokenKind::Integer(integer) => &format!("{}", integer),
-            TokenKind::Float(float) => &format!("{}", float),
+            TokenKind::Integer(integer) => &integer.to_string(),
+            TokenKind::Float(float) => &float.to_string(),
             TokenKind::True => "True",
             TokenKind::False => "False",
             TokenKind::Null => "Null",
             TokenKind::Infinity => "Infinity",
             TokenKind::NaN => "NaN",
 
+            // Group Operators
+            TokenKind::All => "ALL",
+            TokenKind::Some => "Some",
+            TokenKind::Any => "Any",
+
             // Others
             TokenKind::Greater => ">",
             TokenKind::GreaterEqual => ">=",
@@ -328,6 +339,8 @@ fn resolve_symbol_kind(symbol: String) -> TokenKind {
         "cast" => TokenKind::Cast,
         "benchmark" => TokenKind::Benchmark,
 
+        "interval" => TokenKind::Interval,
+
         // Select into
         "into" => TokenKind::Into,
         "outfile" => TokenKind::Outfile,
@@ -365,12 +378,18 @@ fn resolve_symbol_kind(symbol: String) -> TokenKind {
         "and" => TokenKind::AndKeyword,
         "xor" => TokenKind::XorKeyword,
 
+        // Group Operators
+        "all" => TokenKind::All,
+        "some" => TokenKind::Some,
+        "any" => TokenKind::Any,
+
         // True, False and Null
         "true" => TokenKind::True,
         "false" => TokenKind::False,
         "null" => TokenKind::Null,
         "nulls" => TokenKind::Nulls,
 
+        // Infinity and NaN
         "infinity" => TokenKind::Infinity,
         "nan" => TokenKind::NaN,
 
