@@ -8,9 +8,9 @@ use gitql_ast::types::text::TextType;
 use gitql_ast::types::varargs::VarargsType;
 use gitql_core::signature::Signature;
 use gitql_core::signature::StandardFunction;
-use gitql_core::values::base::Value;
 use gitql_core::values::boolean::BoolValue;
 use gitql_core::values::text::TextValue;
+use gitql_core::values::Value;
 
 use crate::meta_types::first_element_type;
 use crate::meta_types::second_element_type;
