@@ -8,10 +8,10 @@ use gitql_ast::types::optional::OptionType;
 use gitql_ast::types::variant::VariantType;
 use gitql_core::signature::Signature;
 use gitql_core::signature::StandardFunction;
-use gitql_core::values::base::Value;
 use gitql_core::values::float::FloatValue;
 use gitql_core::values::integer::IntValue;
 use gitql_core::values::null::NullValue;
+use gitql_core::values::Value;
 
 use crate::meta_types::first_element_type;
 
@@ -164,14 +164,10 @@ pub fn register_std_number_function_signatures(map: &mut HashMap<&'static str, S
 pub fn numeric_abs(inputs: &[Box<dyn Value>]) -> Box<dyn Value> {
     let input_type = inputs[0].data_type();
     if input_type.is_float() {
-        return Box::new(FloatValue {
-            value: inputs[0].as_float().unwrap().abs(),
-        });
+        let value = inputs[0].as_float().unwrap().abs();
+        return Box::new(FloatValue::new(value));
     }
-
-    Box::new(IntValue {
-        value: inputs[0].as_int().unwrap().abs(),
-    })
+    Box::new(IntValue::new(inputs[0].as_int().unwrap().abs()))
 }
 
 pub fn numeric_pi(_inputs: &[Box<dyn Value>]) -> Box<dyn Value> {
