@@ -6,10 +6,10 @@ use gitql_ast::types::text::TextType;
 use gitql_ast::types::variant::VariantType;
 use gitql_core::signature::Signature;
 use gitql_core::signature::StandardFunction;
-use gitql_core::values::base::Value;
 use gitql_core::values::integer::IntValue;
 use gitql_core::values::null::NullValue;
 use gitql_core::values::text::TextValue;
+use gitql_core::values::Value;
 
 use std::collections::HashMap;
 
@@ -388,9 +388,7 @@ pub fn text_left(inputs: &[Box<dyn Value>]) -> Box<dyn Value> {
 
 pub fn text_datalength(inputs: &[Box<dyn Value>]) -> Box<dyn Value> {
     let text = inputs[0].as_text().unwrap();
-    Box::new(IntValue {
-        value: text.as_bytes().len() as i64,
-    })
+    Box::new(IntValue::new(text.len() as i64))
 }
 
 pub fn text_char(inputs: &[Box<dyn Value>]) -> Box<dyn Value> {
